package org.jboss.seam.example.selectitems;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;

import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;

import org.jboss.seam.annotations.Name;

/**
 * @author pmuir
 *
 */

@Entity
@Name("client")
public class Client implements Serializable{
	
	public enum Discount {
		
		TEN_PERCENT("tenPercent"),
		TWENTY_PERCENT("twentyPercent"),
		TWENTY_FIVE_PERCENT("twentyFivePercent");
		
		public static List<Discount> valuesAsList() {
			return Arrays.asList(Discount.values());
		}
		
		private String labelKey;
		
		Discount(String labelKey) {
			this.labelKey = labelKey;
		}
		
		public String getLabelKey() {
			return labelKey;
		}
	}
	
	private static final long serialVersionUID = -2845579466760705228L;

	@Id @GeneratedValue
	private Integer id;

	private String name;
	
	private String address;
	
	private String telephoneNumber;
	
	private String faxNumber;
	
	@ManyToOne
	private ClientType clientType;
	
	@ManyToOne
	private Department department;
	
	@OneToMany
	private List<Factory> factories;
	
	@Enumerated(EnumType.STRING)
	private Discount discount;

	public Client() {
	}

	@Override
	public boolean equals(Object other) {
		if (other instanceof Client) {
			Client that = (Client) other;
			return this.id.intValue() == that.id.intValue();
		} else {
			return false;
		}
	}

	public String getAddress() {
		return address;
	}
    
    public ClientType getClientType() {
		return clientType;
	}
    
    public Discount getDiscount() {
		return discount;
	}

	public String getFaxNumber() {
		return faxNumber;
	}

	public Integer getId() {
		return id;
	}

	public String getName() {
		return name;
	}

	public String getTelephoneNumber() {
		return telephoneNumber;
	}

	@Override
    public int hashCode() {
    	if (id == null) return super.hashCode();
    	return id.hashCode();
    }

	public void setAddress(String address) {
		this.address = address;
	}

	

	public Department getDepartment() {
		return department;
	}

	public void setDepartment(Department branch) {
		this.department = branch;
	}

	public void setClientType(ClientType clientType) {
		this.clientType = clientType;
	}

	public void setDiscount(Discount discount) {
		this.discount = discount;
	}

	public void setFaxNumber(String faxNumber) {
		this.faxNumber = faxNumber;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setTelephoneNumber(String telephoneNumber) {
		this.telephoneNumber = telephoneNumber;
	}

	@Override
    public String toString() {
    	return name + "(" + id + ")";
    }

	public List<Factory> getFactories() {
		return factories;
	}

	public void setFactories(List<Factory> factories) {
		this.factories = factories;
	}
}
