package org.jboss.seam.example.selectitems;

import java.util.List;

import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.faces.application.FacesMessage;
import javax.faces.convert.Converter;
import javax.interceptor.Interceptors;
import javax.persistence.EntityManager;

import org.jboss.seam.annotations.Begin;
import org.jboss.seam.annotations.Create;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.core.FacesMessages;
import org.jboss.seam.example.selectitems.Client.Discount;
import org.jboss.seam.log.Log;
import org.jboss.seam.selectitems.jsf.EntityConverter;

@Stateful
@Name("clientAction")
@Interceptors(org.jboss.seam.ejb.SeamInterceptor.class)
public class ClientActionBean implements ClientAction {
	
	@In(required=false)
	@Out
	private Client client;
	
	private String test;
	
	@In(create=true)
	private EntityManager em;
	
	@Logger
	private Log log;
	
	@In(create=true)
	private FacesMessages facesMessages;
	

	@Create
	@Begin
	public void begin() {		
		client = em.find(Client.class, 1);
	}
	

	@Remove @Destroy
	public void destroy() {
		
	}
	

	@SuppressWarnings("unchecked")
	@Factory("clientTypes")
	public List<ClientType> buildClientTypes() {
		List<ClientType> clientTypes = em.createQuery("select ct from ClientType ct").getResultList();
		log.info("Client type list has " + clientTypes.size() + " entries");
		return clientTypes;
	}
	
	@SuppressWarnings("unchecked")
	@Factory("factories")
	public List<Factory> buildFactories() {
		List<Factory> factories = em.createQuery("select f from Factory f").getResultList();
		log.info("Factory list has " + factories.size() + " entries");
		return factories;
	}
	
	public void apply() {
		try {
			facesMessages.add(FacesMessage.SEVERITY_INFO, "Changes applied");
		} catch (Exception e) {
			facesMessages.add(FacesMessage.SEVERITY_ERROR, "An Exception occurred, look at the console");
			e.printStackTrace();
		}
		
	}

	@SuppressWarnings("unchecked")
	@Factory("departments")
	public List<Department> buildDepartments() {
		List<Department> departments = em.createQuery("select d from Department d").getResultList();
		log.info("Department list has " + departments.size() + " entries");
		return departments;
	}
	
	@Factory("discounts")
	public List<Discount> buildDiscounts() {
		List<Discount> discounts = Client.Discount.valuesAsList();
		log.info("Discount list has " + discounts.size() + " entries");
		return discounts;
	}


	public String getTest() {
		return test;
	}


	public void setTest(String test) {
		this.test = test;
	}
	
	public Converter getDepartmentConverter() {
		return new EntityConverter<Department>() {

			@Override
			protected Object getIdAsObject(String id, Class clazz) {
				String branch = id.substring(0, id.indexOf("_"));
				String manager = id.substring(id.indexOf("_") + 1);
				return new DepartmentId(branch, manager);
			}

			@Override
			protected String getIdAsString(Department entity) {
				return entity.getId().getBranch() + "_" + entity.getId().getManager();
			}
			
		};
	}

}
