package org.jboss.seam.selectitems.jsf;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;

import org.jboss.seam.core.FacesMessages;
import org.jboss.seam.core.Messages;
import org.jboss.seam.selectitems.SelectItemsConfig;

/**
 * Base Converter class for SelectItems.  Takes care of handling 'No Selection' case
 * Override convertToString and convertToObject rather than getAsString and getAsObject
 * @author pmuir
 *
 */
public abstract class SelectItemsConverter implements Converter {
	
	/**
	 * This should be overriden rather than getAsString
	 * @param context
	 * @param component
	 * @param value
	 * @return
	 * @throws ConverterException
	 */
	protected abstract String convertToString(FacesContext context, UIComponent component, Object value) throws ConverterException;
	
	/**
	 * This should be overriden rather than getAsObject
	 * @param context
	 * @param component
	 * @param value
	 * @return
	 * @throws ConverterException
	 */
	protected abstract Object convertToObject(FacesContext context, UIComponent component, String value) throws ConverterException;

	/**
	 * Override convertToObject rather than this
	 */
	public Object getAsObject(FacesContext context, UIComponent component,
			String value) throws ConverterException {
		if (SelectItemsConfig.NO_SELECTION_VALUE.equals(value)) {
			return null;
		} else {
			return convertToObject(context, component, value);
		}
	}

	/**
	 * Override convertToString rather than this
	 */
	public String getAsString(FacesContext context, UIComponent component,
			Object value) throws ConverterException {
		if (value == null || SelectItemsConfig.NO_SELECTION_VALUE.equals(value)) {
			return SelectItemsConfig.NO_SELECTION_VALUE;
		} else {
			return convertToString(context, component, value);
		}
	}
	
	/**
	 * Utitliy method to get a FacesMessage for ConverterExceptions
	 * @param message The message to create
	 * @return The FacesMessage Object
	 */
	protected FacesMessage getMessage(String message) {
		boolean overrideMessage = false;
		String overridenMessage = "";
		try { 
			overridenMessage = (String) Messages.instance().get(message);
			overrideMessage = overridenMessage != null && !overridenMessage.equals(message);
		} catch (Exception e) {
			overrideMessage = false;
		}
		if (overrideMessage) {
			return FacesMessages.createFacesMessage(FacesMessage.SEVERITY_FATAL,
					(String) Messages.instance().get(overridenMessage));
		} else {
			try {
				ResourceBundle bundle = ResourceBundle.getBundle("org.jboss.seam.selectitems.messages");
				String value = bundle.getString(message);
				if (value == null) {
					return FacesMessages.createFacesMessage(FacesMessage.SEVERITY_FATAL,
							message);
				} else {
					return FacesMessages.createFacesMessage(FacesMessage.SEVERITY_FATAL,
							value);
				}
			} catch (MissingResourceException e) {
				throw new ConverterException("Error occured accessing the resource bundle", e);
			}
		}
		
	}
	
	

}
