package org.jboss.seam.selectitems.ui;

import java.util.ArrayList;
import java.util.List;

import javax.faces.FacesException;
import javax.faces.application.Application;
import javax.faces.component.UISelectItems;
import javax.faces.component.ValueHolder;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import javax.faces.model.SelectItem;
import javax.persistence.Entity;

import org.jboss.seam.core.PojoCache;
import org.jboss.seam.selectitems.SelectItemsConfig;
import org.jboss.seam.selectitems.databinding.SelectItemsListModel;
import org.jboss.seam.selectitems.jsf.SelectItemsConverter;

public class UISeamSelectItems extends UISelectItems {
	
	public static final String COMPONENT_TYPE = "org.jboss.seam.selectitems.SelectItems";
	
	private String cacheKey;
	private String cacheRegion;
	private Boolean cache;
	private String noSelectionLabel;
	private Boolean hideNoSelection;
	private String var;
	private String label;
	private String key;
	private Boolean disabled;

	public String getCacheKey() {
		if (cacheKey != null) {
			return cacheKey;
		} else {
			return getString("cacheKey");
		}
	}

	public void setCacheKey(String cacheKey) {
		this.cacheKey = cacheKey;
	}
	
	public String getCacheRegion() {
		if (cacheRegion != null) {
			return cacheRegion;
		} else {
			String value = getString("cacheRegion");
			return value != null ? value : SelectItemsConfig.instance().getCacheRegion();
		}
	}

	public void setCacheRegion(String cacheRegion) {
		this.cacheRegion = cacheRegion;
	}

	public boolean isCache() {
		if (cache != null) {
			return cache;
		} else {
			Boolean value = getBoolean("cache");
			return value != null ? value : SelectItemsConfig.instance().isCache();
		}
	}

	public void setCache(boolean cache) {
		this.cache = cache;
	}
	
	public String getNoSelectionLabel() {
		if (noSelectionLabel != null) {
			return noSelectionLabel;
		} else {
			String value = getString("noSelectionLabel");
			return value != null ? value : SelectItemsConfig.instance().getNoSelectionLabel(); 
		}
	}

	public void setNoSelectionLabel(String noSelectionLabel) {
		this.noSelectionLabel = noSelectionLabel;
	}

	public boolean isHideNoSelection() {
		if (hideNoSelection != null) {
			return hideNoSelection;
		} else {
			Boolean value = getBoolean("hideNoSelection");
			return value != null ? value : SelectItemsConfig.instance().isHideNoSelection();
		}
	}

	public void setHideNoSelection(boolean hideNoSelection) {
		this.hideNoSelection = hideNoSelection;
	}
	
	public String getVar() {
		return var;
	}

	public void setVar(String var) {
		this.var = var;
	}
	
	public String getKey() {
		if (key != null) {
			return key;
		} else {
			return getString("key");
		}
	}

	public void setKey(String key) {
		this.key = key;
	}

	public String getLabel() {
		if (label != null) {
			return label;
		} else {
			return getString("label");
		}
	}

	public void setLabel(String label) {
		this.label = label;
	}
	
	public boolean isDisabled() {
		if (disabled != null) {
			return disabled;
		} else {
			Boolean value = getBoolean("disabled");
			return value != null ? value : false;
		}
	}

	public void setDisabled(boolean disabled) {
		this.disabled = disabled;
	}

	@Override
	public void restoreState(FacesContext context, Object state) {
		Object[] values = (Object[]) state;
		super.restoreState(context, values[0]);
		cacheKey = (String) values[1];
		cacheRegion = (String) values[2];
		cache = (Boolean) values[3];
		noSelectionLabel = (String) values[4];
		hideNoSelection = (Boolean) values[5];
		var = (String) values[6];
		label = (String) values[7];
		key =(String) values[8];
		disabled = (Boolean) values[9];

	}
	
	@Override
	public Object saveState(FacesContext context) {
		Object[] values = new Object[10];
		values[0] = super.saveState(context);
		values[1] = cacheKey;
		values[2] = cacheRegion;
		values[3] = cache;
		values[4] = noSelectionLabel;
		values[5] = hideNoSelection;
		values[6] = var;
		values[7] = label;
		values[8] = key;
		values[9] = disabled;
		return values;
	}
	
	private String getString(String localName) {
		if (getValueBinding(localName) == null) {
			return null;
		} else {
			return (String) getValueBinding(localName).getValue(getFacesContext());
		}
	}
	
	private Boolean getBoolean(String localName) {
		if (getString(localName) != null) {
			return Boolean.valueOf(getString(localName));
		} else {
			return null;
		}
	}
	
	public Object getValue() {
		addConverters();
		if (isCacheEnabled()) {
			Object cachedContent = getFromCache();
			if (cachedContent != null) {
				return cachedContent;
			} else {
				// Execute value binding now and place content in cache
				Object value = createSelectItems(super.getValue());
				cacheValue(value);
				return value;
			}
		} else {
			return createSelectItems(super.getValue());
		}
	}
	
	private Object createSelectItems(Object originalValue) {
		if (super.getValue() instanceof SelectItemsListModel) {
			return originalValue;
		} else {
			// This is @Out style
			if (originalValue instanceof Iterable) {
				List<SelectItem> selectItems = new ArrayList<SelectItem>();
				addNoSelectionLabel(selectItems, originalValue);
				for (Object o : (Iterable) originalValue) {
					initVar(o);
					selectItems.add(new SelectItem(getKey() == null ? o : getKey(), getLabel(), "", isDisabled()));
					destroyVar();
				}
				return selectItems;
			} else {
				throw new IllegalArgumentException("value must be iteratable");
			}
		}

	}
	
	private boolean addNoSelectionLabel(List<SelectItem> selectItems, Object originalValue) {
		boolean add = true;
		if (isHideNoSelection() &&  getParentValue() != null){
			add = false;
		}
		if (add && getNoSelectionLabel() != null) {
			SelectItem s = new SelectItem(SelectItemsConfig.NO_SELECTION_VALUE, getNoSelectionLabel());
			selectItems.add(s);
			return true;
		} else {
			return false;
		}
	}
	
	@SuppressWarnings("unchecked")
	private void initVar(Object varValue) {
		if (getVar() == null) {
			throw new FacesException("var attribute must be set");
		}
		getFacesContext().getExternalContext().getRequestMap().put(
				getVar(), varValue);
	}
	
	private void destroyVar() {
		getFacesContext().getExternalContext().getRequestMap().remove(
				getVar());
	}
	
	@SuppressWarnings("unchecked")
	private List<SelectItem> getFromCache() {
		try {
			return (List<SelectItem>) PojoCache.instance().get(getCacheRegion(), evaluateCacheKey());
		} catch (Exception e) {
			throw new FacesException("error accessing cache", e);
		}
	}
	
	private boolean isCacheEnabled() {
		return evaluateCacheKey() != null;
	}
	
	private String evaluateCacheKey() {
		if (getCacheKey() == null && isCache()) {
			// Work out cache key from value attribute
			ValueBinding vb = this.getValueBinding("value");
			String value = vb.getExpressionString();
			if (value.startsWith("#{") && value.endsWith("}")) {
				this.cacheKey = value.substring(2, value.length() - 1);
			}
		}
		return cacheKey;
	}
	
	private void cacheValue(Object value) {
		try {
			PojoCache.instance().put(getCacheRegion(), evaluateCacheKey(), value);
		} catch (Exception ce) {
			throw new FacesException("Error accessing cache");
		}
	}
	
	private boolean addConverters() {
		// Only add a converter if the user hasn't asked key as the 'value'
		if (getKey() == null) {
			if (this.getParent() instanceof ValueHolder) {
				ValueHolder valueHolder = (ValueHolder) this.getParent();
				// Does an existing converter exist?
				ValueBinding vb = this.getParent().getValueBinding("value");
				if (valueHolder.getConverter() != null) {
					// Ensure the converter implements the SelectItems converter
					if (!(valueHolder.getConverter() instanceof SelectItemsConverter)) {
						throw new FacesException("Converter must implement SelectItemsConverter: " + valueHolder.getConverter() );
					}
				} else if (vb != null && vb.getType(getFacesContext()) != null) {
					Application application = getFacesContext().getApplication();
					Class<?> clazz = vb.getType(getFacesContext());
					if (clazz.isEnum()) {
						valueHolder.setConverter(application
								.createConverter("org.jboss.seam.EnumConverter"));
						return true;
					} else if (clazz.isAnnotationPresent(Entity.class)) {
						valueHolder.setConverter(application
								.createConverter("org.jboss.seam.EntityConverter"));
						return true;
					}
				}
			} else {
				throw new FacesException(
						"parent not instance of ValueHolder: " + this.getParent());
			}
		}
		return false;
	}
	
	private Object getParentValue() {
		if (getParent() instanceof ValueHolder) {
			ValueHolder parent = (ValueHolder) getParent();
			return parent.getValue();
		} else {
			return null;
		}
	}
}